@@ -3,13 +3,13 @@ import math
 import torch
 from torch.optim.optimizer import Optimizer
 
-from pytorch_optimizer.base.base_optimizer import BaseOptimizer
 from pytorch_optimizer.base.exception import NoSparseGradientError
+from pytorch_optimizer.base.optimizer import BaseOptimizer
 from pytorch_optimizer.base.types import BETAS, CLOSURE, DEFAULTS, LOSS, PARAMETERS
 
 
 class AdaBelief(Optimizer, BaseOptimizer):
-    r"""Adapting Step-sizes by the Belief in Observed Gradients
+    r"""Adapting Step-sizes by the Belief in Observed Gradients.
 
     :param params: PARAMETERS. iterable of parameters to optimize or dicts defining parameter groups.
     :param lr: float. learning rate.
@@ -53,15 +53,15 @@ class AdaBelief(Optimizer, BaseOptimizer):
 
         self.validate_parameters()
 
-        defaults: DEFAULTS = dict(
-            lr=lr,
-            betas=betas,
-            eps=eps,
-            weight_decay=weight_decay,
-            amsgrad=amsgrad,
-            adamd_debias_term=adamd_debias_term,
-            buffer=[[None, None, None] for _ in range(10)],
-        )
+        defaults: DEFAULTS = {
+            'lr': lr,
+            'betas': betas,
+            'eps': eps,
+            'weight_decay': weight_decay,
+            'amsgrad': amsgrad,
+            'adamd_debias_term': adamd_debias_term,
+            'buffer': [[None, None, None] for _ in range(10)],
+        }
         super().__init__(params, defaults)
 
     def validate_parameters(self):
@@ -71,7 +71,7 @@ class AdaBelief(Optimizer, BaseOptimizer):
         self.validate_epsilon(self.eps)
 
     @property
-    def __name__(self) -> str:
+    def __str__(self) -> str:
         return 'AdaBelief'
 
     @torch.no_grad()
@@ -94,20 +94,19 @@ class AdaBelief(Optimizer, BaseOptimizer):
                 loss = closure()
 
         for group in self.param_groups:
+            beta1, beta2 = group['betas']
+            weight_decay: float = group['weight_decay']
+
+            if self.rectify:
+                n_sma_max: float = 2.0 / (1.0 - beta2) - 1.0
+
             for p in group['params']:
                 if p.grad is None:
                     continue
 
                 grad = p.grad
                 if grad.is_sparse:
-                    raise NoSparseGradientError(self.__name__)
-
-                if grad.dtype in (torch.float16, torch.bfloat16):
-                    grad = grad.float()
-
-                p_fp32 = p
-                if p.dtype in (torch.float16, torch.bfloat16):
-                    p_fp32 = p_fp32.float()
+                    raise NoSparseGradientError(self.__str__)
 
                 state = self.state[p]
                 if len(state) == 0:
@@ -118,75 +117,65 @@ class AdaBelief(Optimizer, BaseOptimizer):
                         state['max_exp_avg_var'] = torch.zeros_like(p)
 
                 if self.weight_decouple:
-                    decay: float = (
-                        group['lr'] * group['weight_decay'] if not self.fixed_decay else group['weight_decay']
-                    )
-                    p_fp32.mul_(1.0 - decay)
-                elif group['weight_decay'] != 0:
-                    grad.add_(p_fp32, alpha=group['weight_decay'])
-
-                exp_avg, exp_avg_var = state['exp_avg'], state['exp_avg_var']
+                    p.mul_(1.0 - (group['lr'] * weight_decay if not self.fixed_decay else weight_decay))
+                elif weight_decay > 0.0:
+                    grad.add_(p, alpha=weight_decay)
 
                 state['step'] += 1
-                beta1, beta2 = group['betas']
+                exp_avg, exp_avg_var = state['exp_avg'], state['exp_avg_var']
 
                 bias_correction1 = 1.0 - beta1 ** state['step']
-                bias_correction2 = 1.0 - beta2 ** state['step']
+                bias_correction2_sq = math.sqrt(1.0 - beta2 ** state['step'])
 
                 exp_avg.mul_(beta1).add_(grad, alpha=1.0 - beta1)
                 grad_residual = grad - exp_avg
-                exp_avg_var.mul_(beta2).addcmul_(grad_residual, grad_residual, value=1.0 - beta2)
+                exp_avg_var.mul_(beta2).addcmul_(grad_residual, grad_residual, value=1.0 - beta2).add_(group['eps'])
 
-                exp_avg_var = exp_avg_var.add_(group['eps'])
                 if group['amsgrad']:
-                    exp_avg_var = torch.max(state['max_exp_avg_var'], exp_avg_var)
-
-                de_nom = (exp_avg_var.sqrt() / math.sqrt(bias_correction2)).add_(group['eps'])
+                    max_exp_avg_var = state['max_exp_avg_var']
+                    torch.max(max_exp_avg_var, exp_avg_var, out=max_exp_avg_var)
+                    de_nom = max_exp_avg_var.sqrt()
+                else:
+                    de_nom = exp_avg_var.sqrt()
+                de_nom.div_(bias_correction2_sq).add_(group['eps'])
 
                 if not self.rectify:
-                    step_size = group['lr']
-                    if not group['adamd_debias_term']:
-                        step_size /= bias_correction1
-                    p_fp32.addcdiv_(exp_avg, de_nom, value=-step_size)
+                    step_size: float = group['lr'] if group['adamd_debias_term'] else group['lr'] / bias_correction1
+                    p.addcdiv_(exp_avg, de_nom, value=-step_size)
+                    continue
+
+                buffered = group['buffer'][state['step'] % 10]
+                if state['step'] == buffered[0]:
+                    n_sma, step_size = buffered[1], buffered[2]
                 else:
-                    buffered = group['buffer'][state['step'] % 10]
-                    if state['step'] == buffered[0]:
-                        n_sma, step_size = buffered[1], buffered[2]
-                    else:
-                        buffered[0] = state['step']
-                        beta2_t = beta2 ** state['step']
-                        n_sma_max = 2 / (1 - beta2) - 1
-                        n_sma = n_sma_max - 2 * state['step'] * beta2_t / (1 - beta2_t)
-                        buffered[1] = n_sma
-
-                        if n_sma >= self.n_sma_threshold:
-                            rt = math.sqrt(
-                                (1 - beta2_t)
-                                * (n_sma - 4)
-                                / (n_sma_max - 4)
-                                * (n_sma - 2)
-                                / n_sma
-                                * n_sma_max
-                                / (n_sma_max - 2)
-                            )
-
-                            step_size = rt
-                            if not group['adamd_debias_term']:
-                                step_size /= bias_correction1
-                        elif self.degenerated_to_sgd:
-                            step_size = 1.0 / bias_correction1
-                        else:
-                            step_size = -1
-
-                        buffered[2] = step_size
+                    buffered[0] = state['step']
+                    beta2_t = beta2 ** state['step']
+                    n_sma = n_sma_max - 2 * state['step'] * beta2_t / (1 - beta2_t)
+                    buffered[1] = n_sma
 
                     if n_sma >= self.n_sma_threshold:
-                        de_nom = exp_avg_var.sqrt().add_(group['eps'])
-                        p_fp32.addcdiv_(exp_avg, de_nom, value=-step_size * group['lr'])
-                    elif step_size > 0:
-                        p_fp32.add_(exp_avg, alpha=-step_size * group['lr'])
+                        step_size = math.sqrt(
+                            (1 - beta2_t)
+                            * (n_sma - 4)
+                            / (n_sma_max - 4)
+                            * (n_sma - 2)
+                            / n_sma
+                            * n_sma_max
+                            / (n_sma_max - 2)
+                        )
+                        if not group['adamd_debias_term']:
+                            step_size /= bias_correction1
+                    elif self.degenerated_to_sgd:
+                        step_size = 1.0 / bias_correction1
+                    else:
+                        step_size = -1
 
-                if p.dtype in (torch.float16, torch.bfloat16):
-                    p.copy_(p_fp32)
+                    buffered[2] = step_size
+
+                if n_sma >= self.n_sma_threshold:
+                    de_nom = exp_avg_var.sqrt().add_(group['eps'])
+                    p.addcdiv_(exp_avg, de_nom, value=-step_size * group['lr'])
+                elif step_size > 0:
+                    p.add_(exp_avg, alpha=-step_size * group['lr'])
 
         return loss
