@@ -3,14 +3,14 @@ import math
 import torch
 from torch.optim.optimizer import Optimizer
 
-from pytorch_optimizer.base.base_optimizer import BaseOptimizer
 from pytorch_optimizer.base.exception import NoSparseGradientError, ZeroParameterSizeError
+from pytorch_optimizer.base.optimizer import BaseOptimizer
 from pytorch_optimizer.base.types import BETAS, CLOSURE, DEFAULTS, LOSS, PARAMETERS
 from pytorch_optimizer.optimizer.gc import centralize_gradient
 
 
 class Adai(Optimizer, BaseOptimizer):
-    r"""Disentangling the Effects of Adaptive Learning Rate and Momentum
+    r"""Disentangling the Effects of Adaptive Learning Rate and Momentum.
 
     :param params: PARAMETERS. iterable of parameters to optimize or dicts defining parameter groups.
     :param lr: float. learning rate.
@@ -44,13 +44,13 @@ class Adai(Optimizer, BaseOptimizer):
 
         self.validate_parameters()
 
-        defaults: DEFAULTS = dict(
-            lr=lr,
-            betas=betas,
-            weight_decay=weight_decay,
-            dampening=dampening,
-            eps=eps,
-        )
+        defaults: DEFAULTS = {
+            'lr': lr,
+            'betas': betas,
+            'weight_decay': weight_decay,
+            'dampening': dampening,
+            'eps': eps,
+        }
         super().__init__(params, defaults)
 
     def validate_parameters(self):
@@ -60,7 +60,7 @@ class Adai(Optimizer, BaseOptimizer):
         self.validate_epsilon(self.eps)
 
     @property
-    def __name__(self) -> str:
+    def __str__(self) -> str:
         return 'Adai'
 
     @torch.no_grad()
@@ -85,13 +85,14 @@ class Adai(Optimizer, BaseOptimizer):
         exp_avg_sq_hat_sum: float = 0.0
 
         for group in self.param_groups:
+            _, beta2 = group['betas']
             for p in group['params']:
                 if p.grad is None:
                     continue
 
                 grad = p.grad
                 if grad.is_sparse:
-                    raise NoSparseGradientError(self.__name__)
+                    raise NoSparseGradientError(self.__str__)
 
                 param_size += p.numel()
 
@@ -105,20 +106,18 @@ class Adai(Optimizer, BaseOptimizer):
 
                 state['step'] += 1
 
-                exp_avg_sq = state['exp_avg_sq']
-                _, beta2 = group['betas']
-
                 if self.use_gc:
                     grad = centralize_gradient(grad, gc_conv_only=False)
 
                 bias_correction2 = 1.0 - beta2 ** state['step']
 
-                if group['weight_decay'] != 0:
+                if group['weight_decay'] > 0.0:
                     if self.weight_decouple:
                         p.mul_(1.0 - group['lr'] * group['weight_decay'])
                     else:
                         grad.add_(p, alpha=group['weight_decay'])
 
+                exp_avg_sq = state['exp_avg_sq']
                 exp_avg_sq.mul_(beta2).addcmul_(grad, grad, value=1.0 - beta2)
 
                 exp_avg_sq_hat_sum += exp_avg_sq.sum() / bias_correction2
@@ -129,6 +128,8 @@ class Adai(Optimizer, BaseOptimizer):
         exp_avg_sq_hat_mean = exp_avg_sq_hat_sum / param_size
 
         for group in self.param_groups:
+            beta0, beta2 = group['betas']
+            beta0_dp = math.pow(beta0, 1.0 - group['dampening'])
             for p in group['params']:
                 if p.grad is None:
                     continue
@@ -138,7 +139,6 @@ class Adai(Optimizer, BaseOptimizer):
 
                 exp_avg, exp_avg_sq = state['exp_avg'], state['exp_avg_sq']
                 beta1_prod = state['beta1_prod']
-                beta0, beta2 = group['betas']
 
                 bias_correction2 = 1.0 - beta2 ** state['step']
 
@@ -152,7 +152,7 @@ class Adai(Optimizer, BaseOptimizer):
                 bias_correction1 = 1.0 - beta1_prod
 
                 exp_avg.mul_(beta1).addcmul_(beta3, grad)
-                exp_avg_hat = exp_avg / bias_correction1 * math.pow(beta0, 1.0 - group['dampening'])
+                exp_avg_hat = exp_avg / bias_correction1 * beta0_dp
 
                 p.add_(exp_avg_hat, alpha=-group['lr'])
 
