@@ -4,12 +4,12 @@ from typing import Dict
 import torch
 from torch.optim import Optimizer
 
-from pytorch_optimizer.base.base_optimizer import BaseOptimizer
+from pytorch_optimizer.base.optimizer import BaseOptimizer
 from pytorch_optimizer.base.types import CLOSURE, DEFAULTS, LOSS, OPTIMIZER, STATE
 
 
 class Lookahead(Optimizer, BaseOptimizer):
-    r"""k steps forward, 1 step back
+    r"""k steps forward, 1 step back.
 
     :param optimizer: OPTIMIZER. base optimizer.
     :param k: int. number of lookahead steps.
@@ -36,11 +36,13 @@ class Lookahead(Optimizer, BaseOptimizer):
         self.state: STATE = defaultdict(dict)
         self.reset()
 
-        self.defaults: DEFAULTS = dict(
-            k=k,
-            alpha=alpha,
-            pullback_momentum=pullback_momentum,
-            **optimizer.defaults,
+        self.defaults: DEFAULTS = optimizer.defaults
+        self.defaults.update(
+            {
+                'k': k,
+                'alpha': alpha,
+                'pullback_momentum': pullback_momentum,
+            }
         )
 
     def validate_parameters(self):
