@@ -8,13 +8,13 @@ import math
 import torch
 from torch.optim import Optimizer
 
-from pytorch_optimizer.base.base_optimizer import BaseOptimizer
 from pytorch_optimizer.base.exception import NoSparseGradientError
+from pytorch_optimizer.base.optimizer import BaseOptimizer
 from pytorch_optimizer.base.types import CLOSURE, DEFAULTS, LOSS, PARAMETERS
 
 
 class MADGRAD(Optimizer, BaseOptimizer):
-    r"""A Momentumized, Adaptive, Dual Averaged Gradient Method for Stochastic (slightly modified)
+    r"""A Momentumized, Adaptive, Dual Averaged Gradient Method for Stochastic (slightly modified).
 
     :param params: PARAMETERS. iterable of parameters to optimize or dicts defining parameter groups.
     :param lr: float. learning rate.
@@ -42,7 +42,12 @@ class MADGRAD(Optimizer, BaseOptimizer):
 
         self.validate_parameters()
 
-        defaults: DEFAULTS = dict(lr=lr, eps=eps, momentum=momentum, weight_decay=weight_decay)
+        defaults: DEFAULTS = {
+            'lr': lr,
+            'weight_decay': weight_decay,
+            'momentum': momentum,
+            'eps': eps,
+        }
         super().__init__(params, defaults)
 
     def validate_parameters(self):
@@ -52,7 +57,7 @@ class MADGRAD(Optimizer, BaseOptimizer):
         self.validate_epsilon(self.eps)
 
     @property
-    def __name__(self) -> str:
+    def __str__(self) -> str:
         return 'MADGRAD'
 
     @torch.no_grad()
@@ -65,7 +70,7 @@ class MADGRAD(Optimizer, BaseOptimizer):
 
                 state['grad_sum_sq'] = torch.zeros_like(p)
                 state['s'] = torch.zeros_like(p)
-                if group['momentum'] != 0:
+                if group['momentum'] > 0.0:
                     state['x0'] = torch.clone(p).detach()
 
     @torch.no_grad()
@@ -86,7 +91,7 @@ class MADGRAD(Optimizer, BaseOptimizer):
         for group in self.param_groups:
             eps = group['eps']
             lr = group['lr'] + eps
-            decay = group['weight_decay']
+            weight_decay = group['weight_decay']
             momentum = group['momentum']
 
             ck: float = 1.0 - momentum
@@ -102,24 +107,21 @@ class MADGRAD(Optimizer, BaseOptimizer):
                 if 'grad_sum_sq' not in state:
                     state['grad_sum_sq'] = torch.zeros_like(p)
                     state['s'] = torch.zeros_like(p)
-                    if momentum != 0:
+                    if momentum > 0.0:
                         state['x0'] = torch.clone(p).detach()
 
-                if momentum != 0.0 and grad.is_sparse:
-                    raise NoSparseGradientError(self.__name__, note='momentum != 0.0')
+                if momentum > 0.0 and grad.is_sparse:
+                    raise NoSparseGradientError(self.__str__, note='momentum > 0.0')
 
                 grad_sum_sq = state['grad_sum_sq']
                 s = state['s']
 
-                if decay != 0 and not self.decouple_decay:
+                if weight_decay > 0.0 and not self.decouple_decay:
                     if grad.is_sparse:
-                        raise NoSparseGradientError(self.__name__, note='weight_decay')
+                        raise NoSparseGradientError(self.__str__, note='weight_decay')
 
-                    # original implementation
-                    grad.add_(p, alpha=decay)
-
-                    # Apply weight decay - L2 / AdamW style
-                    # p.mul_(1.0 - lr * decay)
+                    # original implementation. not AdamW style
+                    grad.add_(p, alpha=weight_decay)
 
                 if grad.is_sparse:
                     grad = grad.coalesce()
@@ -167,7 +169,7 @@ class MADGRAD(Optimizer, BaseOptimizer):
 
                     s.add_(grad, alpha=_lambda)
 
-                    if decay != 0 and self.decouple_decay:
+                    if weight_decay > 0.0 and self.decouple_decay:
                         p_old = p.clone()
 
                     if momentum == 0.0:
@@ -176,8 +178,8 @@ class MADGRAD(Optimizer, BaseOptimizer):
                         z = x0.addcdiv(s, rms, value=-1)
                         p.mul_(1.0 - ck).add_(z, alpha=ck)
 
-                    if decay != 0 and self.decouple_decay:
-                        p.add_(p_old, alpha=-lr * decay)
+                    if weight_decay > 0.0 and self.decouple_decay:
+                        p.add_(p_old, alpha=-lr * weight_decay)
 
         self.state['k'] += 1
 
