@@ -3,13 +3,13 @@ import math
 import torch
 from torch.optim import Optimizer
 
-from pytorch_optimizer.base.base_optimizer import BaseOptimizer
 from pytorch_optimizer.base.exception import NoSparseGradientError
+from pytorch_optimizer.base.optimizer import BaseOptimizer
 from pytorch_optimizer.base.types import BETAS, CLOSURE, DEFAULTS, LOSS, PARAMETERS
 
 
 class RaLamb(Optimizer, BaseOptimizer):
-    r"""RAdam + LAMB. Large Batch Optimization for Deep Learning
+    r"""RAdam + LAMB. Large Batch Optimization for Deep Learning.
 
     :param params: PARAMETERS. iterable of parameters to optimize or dicts defining parameter groups.
     :param lr: float. learning rate.
@@ -47,15 +47,14 @@ class RaLamb(Optimizer, BaseOptimizer):
 
         self.validate_parameters()
 
-        defaults: DEFAULTS = dict(
-            lr=lr,
-            betas=betas,
-            eps=eps,
-            weight_decay=weight_decay,
-            adamd_debias_term=adamd_debias_term,
-            buffer=[[None, None, None] for _ in range(10)],
-        )
-
+        defaults: DEFAULTS = {
+            'lr': lr,
+            'betas': betas,
+            'weight_decay': weight_decay,
+            'adamd_debias_term': adamd_debias_term,
+            'buffer': [[None, None, None] for _ in range(10)],
+            'eps': eps,
+        }
         super().__init__(params, defaults)
 
     def validate_parameters(self):
@@ -65,7 +64,7 @@ class RaLamb(Optimizer, BaseOptimizer):
         self.validate_epsilon(self.eps)
 
     @property
-    def __name__(self) -> str:
+    def __str__(self) -> str:
         return 'RaLamb'
 
     @torch.no_grad()
@@ -79,18 +78,17 @@ class RaLamb(Optimizer, BaseOptimizer):
                 state['exp_avg_sq'] = torch.zeros_like(p)
 
     @torch.no_grad()
-    def get_gradient_norm(self) -> float:
-        norm_sq: float = 0.0
+    def get_global_gradient_norm(self) -> torch.Tensor:
+        device = self.param_groups[0]['params'][0].device
+
+        global_grad_norm = torch.zeros(1, device=device)
+
         for group in self.param_groups:
             for p in group['params']:
-                if p.grad is None:
-                    continue
+                if p.grad is not None:
+                    global_grad_norm.add_(torch.linalg.norm(p.grad).pow(2))
 
-                norm_sq += torch.linalg.norm(p.grad).cpu().numpy() ** 2
-
-        norm = math.sqrt(norm_sq)
-
-        return norm
+        return torch.sqrt(global_grad_norm) + self.eps
 
     @torch.no_grad()
     def step(self, closure: CLOSURE = None) -> LOSS:
@@ -99,46 +97,37 @@ class RaLamb(Optimizer, BaseOptimizer):
             with torch.enable_grad():
                 loss = closure()
 
-        grad_norm: float = 1.0
+        grad_norm = 1.0
         if self.pre_norm:
-            grad_norm = self.get_gradient_norm()
+            grad_norm = self.get_global_gradient_norm()
 
         for group in self.param_groups:
+            beta1, beta2 = group['betas']
+            n_sma_max: float = 2.0 / (1.0 - beta2) - 1.0
             for p in group['params']:
                 if p.grad is None:
                     continue
 
-                if self.pre_norm:
-                    p.grad /= grad_norm
-
                 grad = p.grad
                 if grad.is_sparse:
-                    raise NoSparseGradientError(self.__name__)
+                    raise NoSparseGradientError(self.__str__)
 
-                if grad.dtype in (torch.float16, torch.bfloat16):
-                    grad = grad.float()
-
-                p_fp32 = p
-                if p.dtype in (torch.float16, torch.bfloat16):
-                    p_fp32 = p_fp32.float()
+                if self.pre_norm:
+                    grad.div_(grad_norm)
 
                 state = self.state[p]
 
                 if len(state) == 0:
                     state['step'] = 0
-                    state['exp_avg'] = torch.zeros_like(p_fp32)
-                    state['exp_avg_sq'] = torch.zeros_like(p_fp32)
-                else:
-                    state['exp_avg'] = state['exp_avg'].type_as(p_fp32)
-                    state['exp_avg_sq'] = state['exp_avg_sq'].type_as(p_fp32)
+                    state['exp_avg'] = torch.zeros_like(p)
+                    state['exp_avg_sq'] = torch.zeros_like(p)
 
+                state['step'] += 1
                 exp_avg, exp_avg_sq = state['exp_avg'], state['exp_avg_sq']
-                beta1, beta2 = group['betas']
 
                 exp_avg.mul_(beta1).add_(grad, alpha=1.0 - beta1)
                 exp_avg_sq.mul_(beta2).addcmul_(grad, grad, value=1.0 - beta2)
 
-                state['step'] += 1
                 buffered = group['buffer'][state['step'] % 10]
 
                 bias_correction1 = 1.0 - beta1 ** state['step']
@@ -148,13 +137,12 @@ class RaLamb(Optimizer, BaseOptimizer):
                 else:
                     buffered[0] = state['step']
                     beta2_t = beta2 ** state['step']
-                    n_sma_max = 2 / (1 - beta2) - 1
                     n_sma = n_sma_max - 2 * state['step'] * beta2_t / (1 - beta2_t)
                     buffered[1] = n_sma
 
                     # more conservative since it's an approximated value
                     if n_sma >= self.n_sma_threshold:
-                        rt = math.sqrt(
+                        step_size = math.sqrt(
                             (1 - beta2_t)
                             * (n_sma - 4)
                             / (n_sma_max - 4)
@@ -163,8 +151,6 @@ class RaLamb(Optimizer, BaseOptimizer):
                             * n_sma_max
                             / (n_sma_max - 2)
                         )
-
-                        step_size = rt
                         if not group['adamd_debias_term']:
                             step_size /= bias_correction1
                     elif self.degenerated_to_sgd:
@@ -174,33 +160,29 @@ class RaLamb(Optimizer, BaseOptimizer):
 
                     buffered[2] = step_size
 
-                if group['weight_decay'] != 0:
-                    p_fp32.add_(p_fp32, alpha=-group['weight_decay'] * group['lr'])
+                if group['weight_decay'] > 0.0:
+                    p.add_(p, alpha=-group['weight_decay'] * group['lr'])
 
-                radam_step = p_fp32.clone()
+                radam_step = p.clone()
                 if n_sma >= self.n_sma_threshold:
                     de_nom = exp_avg_sq.sqrt().add_(group['eps'])
                     radam_step.addcdiv_(exp_avg, de_nom, value=-step_size)
                 else:
                     radam_step.add_(exp_avg, alpha=-step_size)
 
-                radam_step = radam_step.pow(2).sum().sqrt()
-                weight_norm = p.pow(2).sum().sqrt().clamp(0.0, self.clamp)
-                if weight_norm == 0 or radam_step == 0:
-                    trust_ratio = 1.0
-                else:
-                    trust_ratio = weight_norm / radam_step
+                radam_norm = radam_step.norm(2.0)
+                weight_norm = p.norm(2.0).clamp(0.0, self.clamp)
+                trust_ratio: float = (
+                    1.0 if weight_norm == 0 or radam_norm == 0 else weight_norm / (radam_norm + self.eps)
+                )
 
                 state['weight_norm'] = weight_norm
-                state['adam_norm'] = radam_step
+                state['adam_norm'] = radam_norm
                 state['trust_ratio'] = trust_ratio
 
                 if n_sma >= self.n_sma_threshold:
-                    p_fp32.addcdiv_(exp_avg, de_nom, value=-step_size * trust_ratio)
+                    p.addcdiv_(exp_avg, de_nom, value=-step_size * trust_ratio)
                 else:
-                    p_fp32.add_(exp_avg, alpha=-step_size * trust_ratio)
-
-                if p.dtype in (torch.float16, torch.bfloat16):
-                    p.copy_(p_fp32)
+                    p.add_(exp_avg, alpha=-step_size * trust_ratio)
 
         return loss
