@@ -3,52 +3,50 @@ from typing import Dict
 import torch
 from torch.optim.optimizer import Optimizer
 
-from pytorch_optimizer.base.base_optimizer import BaseOptimizer
 from pytorch_optimizer.base.exception import NoClosureError
+from pytorch_optimizer.base.optimizer import BaseOptimizer
 from pytorch_optimizer.base.types import CLOSURE, DEFAULTS, OPTIMIZER, PARAMETERS
 
 
 class SAM(Optimizer, BaseOptimizer):
-    r"""Sharpness-Aware Minimization for Efficiently Improving Generalization
+    r"""Sharpness-Aware Minimization for Efficiently Improving Generalization.
 
     Example:
-        from pytorch_optimizer import SAM
-        ...
-        model = YourModel()
-        base_optimizer =  Ranger21
-        optimizer = SAM(model.parameters(), base_optimizer)
-        ...
-        for input, output in data:
-            # first forward-backward pass
+    -------
+        Here's an example::
 
-            # use this loss for any training statistics
-            loss = loss_function(output, model(input))
-            loss.backward()
-            optimizer.first_step(zero_grad=True)
+            model = YourModel()
+            base_optimizer = Ranger21
+            optimizer = SAM(model.parameters(), base_optimizer)
 
-            # second forward-backward pass
-            # make sure to do a full forward pass
-            loss_function(output, model(input)).backward()
-            optimizer.second_step(zero_grad=True)
+            for input, output in data:
+                # first forward-backward pass
 
-    Alternative Example with a single closure-based step function:
-        from pytorch_optimizer import SAM
-        ...
-        model = YourModel()
-        base_optimizer =  Ranger21
-        optimizer = SAM(model.parameters(), base_optimizer)
+                loss = loss_function(output, model(input))
+                loss.backward()
+                optimizer.first_step(zero_grad=True)
 
-        def closure():
-            loss = loss_function(output, model(input))
-            loss.backward()
-            return loss
-        ...
+                # second forward-backward pass
+                # make sure to do a full forward pass
+                loss_function(output, model(input)).backward()
+                optimizer.second_step(zero_grad=True)
 
-        for input, output in data:
-            loss = loss_function(output, model(input))
-            loss.backward()
-            optimizer.step(closure)
-            optimizer.zero_grad()
+        Alternative example with a single closure-based step function::
+
+            model = YourModel()
+            base_optimizer = Ranger21
+            optimizer = SAM(model.parameters(), base_optimizer)
+
+            def closure():
+                loss = loss_function(output, model(input))
+                loss.backward()
+                return loss
+
+            for input, output in data:
+                loss = loss_function(output, model(input))
+                loss.backward()
+                optimizer.step(closure)
+                optimizer.zero_grad()
 
     :param params: PARAMETERS. iterable of parameters to optimize or dicts defining parameter groups
     :param base_optimizer: Optimizer. base optimizer
@@ -69,7 +67,8 @@ class SAM(Optimizer, BaseOptimizer):
 
         self.validate_parameters()
 
-        defaults: DEFAULTS = dict(rho=rho, adaptive=adaptive, **kwargs)
+        defaults: DEFAULTS = {'rho': rho, 'adaptive': adaptive}
+        defaults.update(kwargs)
         super().__init__(params, defaults)
 
         self.base_optimizer = base_optimizer(self.param_groups, **kwargs)
@@ -79,7 +78,7 @@ class SAM(Optimizer, BaseOptimizer):
         self.validate_rho(self.rho)
 
     @property
-    def __name__(self) -> str:
+    def __str__(self) -> str:
         return 'SAM'
 
     @torch.no_grad()
@@ -124,7 +123,7 @@ class SAM(Optimizer, BaseOptimizer):
     @torch.no_grad()
     def step(self, closure: CLOSURE = None):
         if closure is None:
-            raise NoClosureError(self.__name__)
+            raise NoClosureError(self.__str__)
 
         self.first_step(zero_grad=True)
 
